{% load static %}
{% if expandable %}
<a href="#" title="{{tooltip}}" class="diff-expand-btn"
   data-chunk-index="{{chunk.index}}"
{%  if expand_pos %}
   data-lines-of-context="{{expand_pos.0}},{{expand_pos.1}}"
{%  endif %}>
 <div class="rb-icon {{image_class}}"></div>
{% endif %}
{% if text %}{{text|safe}}{% endif %}
{% if expandable %}
</a>
{% endif %}
