.rb-icon {
  background-repeat: no-repeat;
  display: inline-block;
  overflow: hidden;
  text-indent: -99999px;
  vertical-align: middle;
  background-image: url("../images/icons.png");
  background-image: url('../images/icons.png');
  zoom: 1;
  *display: inline;
}
@media (-webkit-min-device-pixel-ratio: 1.5), (min--moz-device-pixel-ratio: 1.5), (-o-min-device-pixel-ratio: 3/2), (min-resolution: 1.5dppx) {
  .rb-icon {
    background-image: url("../images/icons@2x.png");
    background-size: 155px 109px;
  }
}
/*
 * NOTE: Keep these in the same order as found in the spritesheet.
 */
.rb-icon-issue-dropped {
  background-position: -5px -5px;
  width: 16px;
  height: 16px;
}
.rb-icon-issue-open {
  background-position: -26px -5px;
  width: 16px;
  height: 16px;
}
.rb-icon-issue-resolved {
  background-position: -47px -5px;
  width: 16px;
  height: 16px;
}
.rb-icon-collapse-review {
  background-position: -68px -5px;
  width: 18px;
  height: 18px;
}
.rb-icon-expand-review {
  background-position: -91px -5px;
  width: 18px;
  height: 18px;
}
.rb-icon-collapse {
  background-position: -114px -5px;
  width: 14px;
  height: 14px;
}
.rb-icon-expand {
  background-position: -5px -28px;
  width: 14px;
  height: 14px;
}
.rb-icon-datagrid-comment-draft {
  background-position: -24px -28px;
  width: 16px;
  height: 16px;
}
.rb-icon-datagrid-comment-shipit {
  background-position: -45px -28px;
  width: 16px;
  height: 16px;
}
.rb-icon-datagrid-comment {
  background-position: -66px -28px;
  width: 16px;
  height: 16px;
}
.rb-icon-shipit-checkmark {
  background-position: -87px -28px;
  width: 9px;
  height: 10px;
}
.rb-icon-datagrid-new-updates {
  background-position: -101px -28px;
  width: 18px;
  height: 16px;
}
.rb-icon-edit {
  background-position: -124px -28px;
  width: 20px;
  height: 16px;
}
.rb-icon-delete {
  background-position: -5px -49px;
  width: 12px;
  height: 12px;
}
.rb-icon-diff-collapse-chunk {
  background-position: -22px -49px;
  width: 14px;
  height: 14px;
}
.rb-icon-diff-expand-above {
  background-position: -41px -49px;
  width: 28px;
  height: 14px;
}
.rb-icon-diff-expand-all {
  background-position: -74px -49px;
  width: 14px;
  height: 14px;
}
.rb-icon-diff-expand-below {
  background-position: -93px -49px;
  width: 28px;
  height: 14px;
}
.rb-icon-diff-expand-header {
  background-position: -126px -49px;
  width: 14px;
  height: 14px;
}
.rb-icon-shipit {
  background-position: -5px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-admin-enabled {
  background-position: -5px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-admin-disabled {
  background-position: -26px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-admin-add {
  background-position: -47px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-star-off {
  background-position: -68px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-star-on {
  background-position: -89px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-warning {
  background-position: -110px -68px;
  width: 16px;
  height: 16px;
}
.rb-icon-search {
  background-position: -129px -66px;
  width: 20px;
  height: 20px;
}
.rb-icon-search-dark {
  background-position: -3px -86px;
  width: 20px;
  height: 20px;
}
.rb-icon-admin-collapse {
  background-position: -25px -89px;
  width: 20px;
  height: 13px;
}
.rb-icon-admin-expand {
  background-position: -50px -89px;
  width: 20px;
  height: 13px;
}
.rb-icon-jump-to {
  background-position: -76px -86px;
  width: 21px;
  height: 18px;
}
.rb-icon-range-slider {
  background-position: -100px -87px;
  width: 9px;
  height: 18px;
}
.rb-icon-download {
  background-position: -114px -90px;
  width: 14px;
  height: 14px;
}
