{% load djblets_utils %}
{% definevar "precss" %}white-space: pre-wrap; white-space: -moz-pre-wrap; white-space: -pre-wrap; white-space: -o-pre-wrap; word-wrap: break-word;{% enddefinevar %}
{% if not embedded %}
<html>
 <body>
  <div style="font-family: Verdana, Arial, Helvetica, Sans-Serif;">
   <table bgcolor="#f9f3c9" width="100%" cellpadding="12" style="border: 1px #c9c399 solid; border-radius: 6px; -moz-border-radius: 6px; -webkit-border-radius: 6px;">
    <tr>
     <td>
      This is an automatically generated e-mail. To reply, visit:
      <a href="{{domain_method}}://{{domain}}{{review_request.get_absolute_url}}">{{domain_method}}://{{domain}}{{review_request.get_absolute_url}}</a>
     </td>
    </tr>
   </table>
   <br />
{% endif %}
{% block content %}{% endblock %}
{% block footer %}{% endblock %}
{% if not embedded %}
  </div>
 </body>
</html>
{% endif %}
