@import "defs.less";


.dnd-overlay {
  .opacity(80);
  position: fixed;
  left: 0;
  top: 0;
  width: 100%;
  height: 100%;
  background: black;
  color: #DDDDDD;
  font-size: 200%;
  font-weight: bold;
  text-align: center;
  vertical-align: middle;
  z-index: @z-index-page-overlay;
}
