{% extends "base.html" %}

{% load djblets_deco %}

{% block title %}Permission denied.{% endblock %}

{% block content %}
{% box "important" %}
<h1>You do not have permission to view this item.</h1>
<p>TODO: write some nice text.</p>
{% endbox %}
{% endblock %}
