/*
 * The trophy appears too high on IE and the box has padding that I can't
 * get rid of. Just make the trophy inline.
 */
.box.yay img {
  float: left;
  left: 0;
  top: 0;
  margin-right: 10px;
}

/* There's a double border on IE, so remove the top header on columns. */
.datagrid-headers th {
  border-top: 0;
}

/* Line number column and side-by-side divider borders. */
table.sidebyside tr td {
  border-left: 1px #C0C0C0 solid;
}

table.sidebyside tbody.insert td:first-child {
  border-top: 1px #AAAAAA solid;
}

.banner input {
  vertical-align: middle;
}

.box-container {
  width: 99.5%;
}

.box.important {
  padding: 10px;
}

.box.important .box-inner {
  width: 90%;
}

.review-request #details td {
  padding: 4px;
}

/* Position the "View Diff" and other controls correctly. */
.review-request #actions {
  padding: 0 1px 0 0;
}

/*
 * Fix the li spacing bug in IE6 by adding a border -
 * http://blog.onthafly.com/2007/11/ie6-menu-list-item-spacing-nightmare.html
 */
.review-request #actions ul.menu li {
  border-bottom: 1px #dfb16a solid;
}

.screenshot-container .screenshot {
  height: auto;
}

#comment-detail #review_comments ul li h2 .actions a {
  border: 1px black solid !important;
}

/* IE can't seem to do fixed height for the two columns, so force a minimum. */
#dashboard-main .datagrid-main {
  height: 35em;
}

#navbar-outer {
  width: 99.9%;
}

* html .clearfix {
  display: inline-block !important;
}
