{% load i18n %}
<table class="widget-rows" style="width: 100%;">
 <tr>
  <th>{% trans "Comments" %}</th>
  <td>{{widget.data.count_comments}}</td>
 </tr>
 <tr>
  <th>{% trans "Reviews" %}</th>
  <td>{{widget.data.count_reviews}}</td>
 </tr>
 <tr>
  <th>{% trans "Request Drafts" %}</th>
  <td>{{widget.data.count_reviewdrafts}}</td>
 </tr>
 <tr>
  <th>{% trans "Screenshots" %}</th>
  <td>{{widget.data.count_screenshots}}</td>
 </tr>
 <tr>
  <th>{% trans "Attachments" %}</th>
  <td>{{widget.data.count_attachments}}</td>
 </tr>
 <tr>
  <th>{% trans "DiffSets" %}</th>
  <td>{{widget.data.count_diffsets}}</td>
 </tr>
</table>
