{% extends "base.html" %}
{% load djblets_deco i18n %}

{% block title %}{% trans "Password Reset Successful" %}{% endblock %}

{% block content %}
{%  box "passwordreset" %}
 <div class="box-head">
  <h1 class="box-title">{% trans "Password reset successful" %}</h1>
 </div>
 <div class="box-main">
  <p>{% blocktrans %}We've e-mailed you instructions for setting your password
  to the e-mail address you submitted. You should be receiving it shortly.
  {% endblocktrans %}</p>
 </div>
{%  endbox %}
{% endblock %}
