@import (reference) "defs.less";


#search_results {
  margin: 3em;

  .paginator {
    border-top: 1px #E0E0E0 solid;
    font-size: 120%;
    margin-top: 2em;
    padding-top: 2em;

    a {
      color: blue;
      text-decoration: none;
    }

    a, b {
      margin-right: 0.5em;
    }
  }

  .search-results-filters {
    @filter-border-color: #CCC;

    border-bottom: 1px @filter-border-color solid;
    margin-bottom: 2em;

    a {
      color: #666;
      display: inline-block;
      font-size: 130%;
      text-decoration: none;
      padding: 0.5em 1em;
      border: 1px transparent solid;
      border-top-width: 2px;

      &.active {
        color: black;
        border: 1px @filter-border-color solid;
        border-bottom: 1px white solid;
        border-top-width: 2px;
      }
    }
  }

  .search-results-stats {
    color: #444444;
    font-size: 110%;
    padding-bottom: 2em;
    margin-left: 1em;
  }

  .search-result {
    .clearfix();
    line-height: 1.5;
    margin-bottom: 2em;

    a {
      text-decoration: none;
    }

    h2 {
      font-size: 140%;
      font-weight: normal;
      margin: 0;
    }

    .search-result-by {
      color: #555555;
      font-size: 85%;
      margin-top: 0.3em;
    }

    .search-result-body {
      float: left;
    }

    .search-result-excerpt {
      margin-top: 0.3em;
      width: 60em;
    }

    .search-result-icon {
      float: left;
      margin-right: 1em;
    }

    .search-result-links {
      margin-top: 1em;
    }

    .search-result-username {
      font-size: 120%;
    }

    .search-result-user-groups {
      margin-top: 1em;
    }
  }
}
