from __future__ import unicode_literals

SEQUENCE = [
    'is_private',
    'timezone',
    'open_an_issue',
    'extra_data',
    'timezone_length_30',
    'localsiteprofile_permissions',
    'unique_together_baseline',
    'profile_show_closed',
    'profile_should_send_email',
    'profile_should_send_own_updates',
    'profile_default_use_rich_text',
]
