/* RB Colors */
/*
 * At least on Chrome, using points instead of pixels does not reliably
 * result in identical sizings when there are elements positioned inside
 * a <pre>. Pixels fixes this.
 */
/* Diff styles */
/* !(*%!(&^ (see http://www.positioniseverything.net/easyclearing.html) */
.clearfix {
  display: inline-block;
}
.clearfix:after {
  clear: both;
  content: ".";
  display: block;
  height: 0;
  visibility: hidden;
}
/* Hides from IE-mac \*/
* html .clearfix {
  height: 1%;
}
.clearfix {
  display: block;
}
/* End hide from IE-mac */
/****************************************************************************
 * General text-based review styles
 ****************************************************************************/
.text-review-ui .text-review-ui-header {
  border-bottom: 1px #cacaca solid;
  background-color: #f0f0f0;
  background-repeat: no-repeat;
  background-image: -webkit-gradient(linear, 0 0, 0 100%, from(#ffffff), color-stop(50%, #f0f0f0), to(#f0f0f0));
  background-image: -webkit-linear-gradient(#ffffff, #f0f0f0 50%, #f0f0f0);
  background-image: -moz-linear-gradient(top, #ffffff, #f0f0f0 50%, #f0f0f0);
  background-image: -ms-linear-gradient(#ffffff, #f0f0f0 50%, #f0f0f0);
  background-image: -o-linear-gradient(#ffffff, #f0f0f0 50%, #f0f0f0);
  background-image: linear-gradient(#ffffff, #f0f0f0 50%, #f0f0f0);
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffff', endColorstr='#f0f0f0', GradientType=0);
}
.text-review-ui .text-review-ui-header h1 {
  margin: 0;
  padding: 8px;
}
.text-review-ui .text-review-ui-header .text-review-ui-views {
  list-style: none;
  margin: 1em 0 0 5em;
  padding: 0;
}
.text-review-ui .text-review-ui-header .text-review-ui-views li {
  background: #f5f5f5;
  cursor: pointer;
  display: inline-block;
  margin: 0;
  border: 1px #cacaca solid;
  border-bottom: 0;
  vertical-align: bottom;
  -moz-border-radius: 6px 6px 0 0;
  -webkit-border-radius: 6px 6px 0 0;
  border-radius: 6px 6px 0 0;
}
.text-review-ui .text-review-ui-header .text-review-ui-views li.active {
  background: #fdfdfd;
}
.text-review-ui .text-review-ui-header .text-review-ui-views li.active a {
  padding: 6px 12px;
}
.text-review-ui .text-review-ui-header .text-review-ui-views li a {
  color: black;
  display: block;
  padding: 4px 12px;
  text-decoration: none;
}
.text-review-ui .text-review-ui-text-table,
.text-review-ui .text-review-ui-rendered-table {
  -moz-border-radius: 0 0 5px 5px;
  -webkit-border-radius: 0 0 5px 5px;
  border-radius: 0 0 5px 5px;
}
.text-review-ui .text-review-ui-text-table tr:last-child td,
.text-review-ui .text-review-ui-rendered-table tr:last-child td {
  -moz-border-radius: 0 0 5px 0;
  -webkit-border-radius: 0 0 5px 0;
  border-radius: 0 0 5px 0;
}
.text-review-ui .text-review-ui-text-table tr:last-child th,
.text-review-ui .text-review-ui-rendered-table tr:last-child th {
  -moz-border-radius: 0 0 0 5px;
  -webkit-border-radius: 0 0 0 5px;
  border-radius: 0 0 0 5px;
}
.text-review-ui .text-review-ui-text-table .filename-row,
.text-review-ui .text-review-ui-rendered-table .filename-row {
  border-bottom: 1px #cacaca solid;
}
.text-review-ui-comment-thumbnail {
  background: inherit;
}
.text-review-ui-comment-thumbnail td {
  background: white;
}
.text-review-ui-comment-thumbnail td,
.text-review-ui-comment-thumbnail th {
  border-right: 1px #DDD solid;
}
.text-review-ui-comment-thumbnail tr:first-child td {
  border-top: 1px #DDD solid;
}
.text-review-ui-comment-thumbnail tr:last-child td {
  border-bottom: 1px #DDD solid;
}
/****************************************************************************
 * Markdown-specific styles
 ****************************************************************************/
.markdown-review-ui td {
  padding-left: 2px;
}
.markdown-review-ui td * {
  margin: 0;
  padding: 0;
}
.markdown-review-ui td a {
  color: blue;
  text-decoration: underline;
}
.markdown-review-ui td h1 {
  font-size: 2em;
}
.markdown-review-ui td ul,
.markdown-review-ui td ol {
  padding-left: 2em;
}
