<li class="section{% if active %} active{% endif %} {% if url %}has-url{% endif %}{% if css_classes %} {{css_classes|join:' '}}{% endif %}"{% if url %} onclick="javascript:window.location=&quot;{{url}}&quot;; return false;"{% endif %}>
 <div class="datagrid-sidebar-row">
  <span class="label">{% if url %}<a href="{{url}}">{% endif %}{% if icon_name %}<div class="rb-icon {{icon_name}}"></div> {% endif %}{{label}}{% if url %}</a>{% endif %}</span>
{% if has_count %}
  <span class="count {% if count == 0 %}count-zero{% endif %}">{{count}}</span>
{% endif %}
 </div>
{% if items %}
 <ul>
{% for item in items %}
{{item.render}}
{% endfor %}
 </ul>
{% endif %}
</li>
