{% load djblets_utils %}
 <li><a{% attr "id" %}{{action.id}}{% endattr %} href="{{action.url}}">{% if action.image %}<img src="{{action.image}}" width="{{action.image_width}}" height="{{action.image_height}}" border="0" alt="" />{% endif %}{{action.label}}</a></li>
