from __future__ import unicode_literals

SEQUENCE = [
    'file_attachment_orig_filename',
    'file_attachment_file_max_length_512',
    'file_attachment_repo_info',
    'file_attachment_repo_path_no_index',
    'file_attachment_repo_revision_max_length_64',
]
