from __future__ import unicode_literals


SEQUENCE = [
    'add_parent_diffs',
    'filediff_filenames_1024_chars',
    'diffset_basedir',
    'filediff_status',
    'add_diff_hash',
    'diffsethistory_diff_updated',
    'filediffdata_line_counts',
    'diffset_base_commit_id',
    'filediffdata_extra_data',
    'all_extra_data',
]
