{% load i18n %}
<p>
{% blocktrans with rbtools_url="https://www.reviewboard.org/docs/rbtools/dev/" %}
 To configure this repository with <a href="{{rbtools_url}}">RBTools</a>, save the
 following in a <tt>.reviewboardrc</tt> file in the root of your checked out
 working copy and commit it:
{% endblocktrans %}
</p>
<pre>
REVIEWBOARD_URL = '{{reviewboard_url}}'
REPOSITORY = '{{repository.name}}'
</pre>
<p>
{% blocktrans %}
RBTools will then be able to find the correct server and repository when any
developer posts changes using <tt>rbt post</tt>.
{% endblocktrans %}
</p>
<p>
{% blocktrans with config_docs_url="https://www.reviewboard.org/docs/rbtools/dev/rbt/configuration/" %}
See the <a href="{{config_docs_url}}">documentation</a> on configuring
repositories in RBTools for more information.
{% endblocktrans %}
</p>
