RB.APIErrors = {
    NO_ERROR: 0,
    SERVICE_NOT_CONFIGURED: 1,
    DOES_NOT_EXIST: 100,
    PERMISSION_DENIED: 101,
    INVALID_ATTRIBUTE: 102,
    NOT_LOGGED_IN: 103,
    LOGIN_FAILED: 104,
    INVALID_FORM_DATA: 105,
    MISSING_ATTRIBUTE: 105,
    ENABLE_EXTENSION_FAILED: 107,
    DISABLE_EXTENSION_FAILED: 108,
    EXTENSION_INSTALLED: 109,
    INSTALL_EXTENSION_FAILED: 110,
    UNSPECIFIED_DIFF_REVISION: 200,
    INVALID_DIFF_REVISION: 201,
    INVALID_ACTION: 202,
    INVALID_CHANGE_NUMBER: 203,
    CHANGE_NUMBER_IN_USE: 204,
    MISSING_REPOSITORY: 205,
    INVALID_REPOSITORY: 206,
    REPO_FILE_NOT_FOUND: 207,
    INVALID_USER: 208,
    REPO_NOT_IMPLEMENTED: 209,
    REPO_INFO_ERROR: 210,
    NOTHING_TO_PUBLISH: 211,
    EMPTY_CHANGESET: 212,
    SERVER_CONFIG_ERROR: 213,
    BAD_HOST_KEY: 214,
    UNVERIFIED_HOST_KEY: 215,
    MISSING_USER_KEY: 217,
    REPO_AUTHENTICATION_ERROR: 218,
    DIFF_EMPTY: 219,
    DIFF_TOO_BIG: 220,
    FILE_RETRIEVAL_ERROR: 221,
    HOSTINGSVC_AUTH_ERROR: 222,
    GROUP_ALREADY_EXISTS: 223,
    DIFF_PARSE_ERROR: 224
};
