/* RB Colors */
/*
 * At least on Chrome, using points instead of pixels does not reliably
 * result in identical sizings when there are elements positioned inside
 * a <pre>. Pixels fixes this.
 */
/* Diff styles */
/* !(*%!(&^ (see http://www.positioniseverything.net/easyclearing.html) */
.clearfix {
  display: inline-block;
}
.clearfix:after {
  clear: both;
  content: ".";
  display: block;
  height: 0;
  visibility: hidden;
}
/* Hides from IE-mac \*/
* html .clearfix {
  height: 1%;
}
.clearfix {
  display: block;
}
/* End hide from IE-mac */
#admin-actions {
  background: #f0f0f2;
  border-right: 0.1em #888a85 solid;
  border-bottom: 0.1em #888a85 solid;
  float: left;
  margin: 0;
  padding: 0;
  min-height: 100%;
  width: 22.9em;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
  z-index: 130;
}
#admin-actions a {
  color: black;
}
#admin-actions li,
#admin-actions td,
#admin-actions th {
  margin: 0;
  padding: 0.5em 0;
}
#admin-actions .widget-content ul {
  margin: 0;
  padding: 0;
}
#admin-actions .widget-content ul li:first-child {
  padding-top: 0;
}
#admin-actions .widget-content ul li:last-child {
  padding-bottom: 0;
}
#admin-actions .admin-widget {
  border: 0;
  -moz-border-radius: 0;
  -webkit-border-radius: 0;
  border-radius: 0;
  -moz-box-shadow: none;
  -webkit-box-shadow: none;
  box-shadow: none;
  margin: 0;
}
#admin-actions .admin-widget .widget-content {
  padding: 1em;
}
#admin-actions .admin-widget .widget-content table {
  border-collapse: collapse;
}
#admin-actions .admin-widget .widget-content table tr {
  border-bottom: 0;
}
#admin-actions .admin-widget .widget-content li {
  border-bottom: 0;
}
#admin-actions .admin-widget .widget-heading {
  background: #c6dcf3;
  border-top: 1px #888a85 solid;
  border-bottom: 1px #888a85 solid;
  padding-left: 1em;
  -moz-border-radius: 0;
  -webkit-border-radius: 0;
  border-radius: 0;
}
#admin-actions .admin-widget:first-child .widget-heading {
  border-top: 0;
}
#admin-actions .admin-version {
  font-weight: bold;
  margin-top: 1em;
}
#admin-dashboard {
  width: 100%;
  overflow: hidden;
}
#admin-dashboard #admin-widgets {
  float: left;
  margin: 0;
}
#admin-dashboard .admin-sidebar {
  border: none;
}
#admin-extras {
  float: left;
}
#admin-extras .admin-widget {
  float: left;
}
#admin-extras .admin-widget .widget-actions a {
  border-right: none;
  border-left: 1px solid #cccccc;
  float: right;
}
#admin-extras .admin-widget .widget-content .no-result {
  border: none;
  font-size: 13px;
  margin: 0;
}
.admin-widget {
  border: 1px solid #888a85;
  margin: 0 0 20px;
  -moz-border-radius: 6px;
  -webkit-border-radius: 6px;
  border-radius: 6px;
  -moz-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  -webkit-box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
  box-shadow: 1px 1px 2px rgba(0, 0, 0, 0.15);
}
.admin-widget.widget-large {
  width: 460px;
  background-color: white;
}
.admin-widget.widget-small {
  float: left;
  width: 200px;
  margin-left: 20px;
  background-color: white;
}
.admin-widget.widget-hidden .widget-heading {
  -moz-border-radius: 5px;
  -webkit-border-radius: 5px;
  border-radius: 5px;
}
.admin-widget.widget-hidden.widget-has-actions .widget-heading {
  -moz-border-radius: 5px 5px 0 0;
  -webkit-border-radius: 5px 5px 0 0;
  border-radius: 5px 5px 0 0;
}
.admin-widget.widget-hidden .widget-content {
  display: none;
}
.admin-widget.widget-collapsable .widget-heading .btn-state {
  cursor: pointer;
}
.admin-widget .widget-heading {
  background: #a2bedc;
  border-bottom: 1px solid #728eac;
  background-image: url("../images/title_box_top_bg.png");
  color: black;
  font-size: 11px;
  font-weight: 400;
  margin: 0;
  padding: 5px 10px 5px 5px;
  text-transform: uppercase;
  -moz-border-radius: 5px 5px 0 0;
  -webkit-border-radius: 5px 5px 0 0;
  border-radius: 5px 5px 0 0;
}
.admin-widget .widget-heading h1 {
  margin: 0;
}
.admin-widget .widget-heading img {
  float: left;
  margin: 1px 4px 0 0;
  overflow: hidden;
}
.admin-widget .widget-heading .btn-state {
  float: right;
}
.admin-widget .widget-content {
  position: relative;
}
.admin-widget .widget-content caption {
  text-align: left;
  margin-left: 10px;
}
.admin-widget .widget-content ul {
  margin: 0;
  padding: 0;
}
.admin-widget .widget-content ul li {
  border-bottom: 1px solid #f5f5f5;
  list-style-type: none;
  padding: 5px 0;
}
.admin-widget .widget-content ul li:last-child {
  border-bottom: 0;
}
.admin-widget .widget-content ul li a {
  display: block;
}
.admin-widget .widget-content p {
  padding: 10px;
}
.admin-widget .widget-content tr {
  border-bottom: 1px solid #f5f5f5;
}
.admin-widget .widget-content tr:last-child {
  border-bottom: 0;
}
.admin-widget .widget-content tr th {
  color: #4b4b4b;
  font-size: 11px;
  font-weight: normal;
}
.admin-widget .widget-content .no-result {
  border: 4px dashed #CCC;
  color: #888a85;
  font-weight: bold;
  font-size: 14px;
  text-transform: capitalize;
  line-height: 1.9em;
  margin: 15px;
  padding: 15px;
  text-align: center;
}
.admin-widget .widget-content .widget-large-table {
  width: 100%;
}
.admin-widget .widget-content .widget-large-table .visible-column {
  text-align: center;
}
.admin-widget .widget-actions {
  background: url("../images/admin/header_bg.png") repeat-x scroll left bottom transparent;
  border-bottom: 1px solid #cccccc;
  border-right: 1px solid #cccccc;
  border-top: 1px solid #cccccc;
  color: black;
  overflow: hidden;
  padding: 0;
  text-align: left;
  white-space: nowrap;
  -moz-border-radius: 0 0 5px 5px;
  -webkit-border-radius: 0 0 5px 5px;
  border-radius: 0 0 5px 5px;
}
.admin-widget .widget-actions a {
  border-right: 1px solid #cccccc;
  color: black;
  display: block;
  float: left;
  font-size: 12px;
  height: 20px;
  padding: 6px 5px 0;
  text-decoration: none;
}
.admin-widget .widget-actions a:hover {
  background: url("../images/admin/header_bg_primary.png") repeat-x scroll left bottom transparent;
  text-decoration: underline;
}
.admin-widget .widget-actions a.btn-right {
  float: right;
  border-right: none;
  border-left: 1px solid #cccccc;
}
.admin-widget .btn-s {
  opacity: 0.4;
  filter: alpha(opacity=40);
  -moz-opacity: 0.4;
}
.admin-widget .btn-s-checked {
  opacity: 1;
  filter: alpha(opacity=100);
  -moz-opacity: 1;
}
.clear {
  clear: both;
}
/**************************************************************************
 * Per-widget styles
 **************************************************************************/
#database-stats-widget td {
  text-align: right;
}
/* Review Board News widget */
#news-widget .entry {
  border-bottom: 1px solid #888a85;
  padding: 5px;
}
#news-widget .entry h3 {
  font-size: 10px;
  font-weight: normal;
  margin: 0;
  padding: 0;
}
#news-widget .entry h3 a {
  display: block;
}
/* Recent Actions widget */
#recent-actions-widget li {
  background-position: 7px 11px;
  padding: 2px 0 0 25px;
}
#recent-actions-widget li span {
  display: block;
  clear: both;
}
/* Request Statuses widget */
#review-request-statuses-widget #requests-percentages {
  margin-top: 5px;
}
#review-request-statuses-widget #requests-percentages .request-percentage-label {
  font-size: 10px;
  padding: 2px;
  text-align: center;
}
/* Server Cache widget */
#server-cache-widget td,
#server-cache-widget th {
  font-size: 10px;
}
/* Review Board Activity widget */
#activity-graph-widget .widget-content {
  margin: 10px 0 10px 10px;
}
#activity-graph-widget .widget-content .legendLabel {
  font-size: 10px;
}
#activity-graph-widget .widget-content .legend-btn {
  position: absolute;
  left: 30px;
  top: 10px;
  display: none;
  cursor: pointer;
}
/* User Activity widget */
#user-count {
  border-top: 1px solid #888a85;
  bottom: 10px;
  color: #545454;
  padding-left: 30px;
  padding-top: 5px;
  position: absolute;
  right: 20px;
  text-align: left;
  width: 99px;
}
#widget-manage table {
  width: 100%;
}
#widget-manage table .count {
  color: #4b4b4b;
  padding-right: 1em;
  text-align: right;
}
#widget-manage .addlink {
  display: block;
  padding: 0;
  white-space: nowrap;
}
#widget-manage .addlink .rb-icon {
  margin-left: 2px;
}
/* System Information sidebar widget */
#widget-system-info .rb-icon {
  float: right;
}
#widget-system-info .rb-icon .admin-version {
  text-align: right;
}
#widget-sys-settings a.active {
  font-weight: bold;
}
#widget-sys-settings li {
  border: none;
}
