{% load i18n %}
<div class="box-head">
 <h1 class="box-title">{% trans "A database error occurred" %}</h1>
</div>
<div class="box-main">
 <p>
{% blocktrans %}
  There was an error in accessing data from the database. This could be
  a configuration error, or a problem communicating with the database.
  If this was previously working, please be sure that your database is
  running and is not locked.
{% endblocktrans %}
 </p>
 <p>
{% blocktrans %}
  The error is as follows:
{% endblocktrans %}
 </p>
 <pre>
    {{error}}
</pre>
</div>
