{% load i18n staticfiles %}

<div class="news-content-widget">
 <img src="{% static "rb/images/spinner.gif" %}"
      class="loading-indicator" width="16" height="16" border="0" alt="" />
 {% trans "Loading..." %}
</div>
<script type="text/javascript">
$(document).ready(function () {
    // RSS Feed Controls
    $("a.reload-news").click(function(evt) {
        $(".news-content-widget").load("feed/news/?reload=1");
            evt.preventDefault();
        });

    $(".news-content-widget").load("feed/news/", function() {
        $('#admin-extras').masonry('reload');
    });
});
</script>
