/* RB Colors */
/*
 * At least on Chrome, using points instead of pixels does not reliably
 * result in identical sizings when there are elements positioned inside
 * a <pre>. Pixels fixes this.
 */
/* Diff styles */
/* !(*%!(&^ (see http://www.positioniseverything.net/easyclearing.html) */
.clearfix {
  display: inline-block;
}
.clearfix:after {
  clear: both;
  content: ".";
  display: block;
  height: 0;
  visibility: hidden;
}
/* Hides from IE-mac \*/
* html .clearfix {
  height: 1%;
}
.clearfix {
  display: block;
}
/* End hide from IE-mac */
/****************************************************************************
 * Dashboard
 ****************************************************************************/
#dashboard_page {
  overflow: hidden;
  padding-bottom: 3px;
}
#dashboard_page #dashboard_container {
  display: none;
}
#dashboard_container {
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}
#dashboard_container .box-container,
#dashboard_container .box,
#dashboard_container .box-inner,
#dashboard_container #dashboard-wrapper {
  margin-bottom: 0;
  height: 100%;
}
#dashboard-main {
  background: white;
  border-left: 1px #888a85 solid;
  height: 100%;
  vertical-align: top;
  margin-left: 18em;
  -moz-border-radius: 0 5px 5px 0;
  -webkit-border-radius: 0 5px 5px 0;
  border-radius: 0 5px 5px 0;
}
#dashboard-main table:after {
  clear: both;
}
#dashboard-main .datagrid td.bugs a {
  text-decoration: underline;
  color: #0000c0;
}
#dashboard-main .datagrid-body-container {
  overflow-y: auto;
  margin-right: 1px;
}
#dashboard-main .datagrid-titlebox {
  -moz-border-radius: 0 5px 0 0;
  -webkit-border-radius: 0 5px 0 0;
  border-radius: 0 5px 0 0;
}
#dashboard-main .datagrid-title {
  background: transparent;
}
#dashboard-main .datagrid-wrapper {
  height: 100%;
}
#dashboard-main .main {
  padding: 0;
}
#dashboard-main .repository-column {
  white-space: nowrap;
}
#dashboard-main .diff-size-column {
  font-size: 90%;
}
#dashboard-main .diff-size-column.delete {
  color: darkred;
}
#dashboard-main .diff-size-column.insert {
  color: darkgreen;
}
#dashboard_actions {
  background: #333333;
  color: white;
  position: absolute;
  top: 0;
  bottom: 0;
  padding: 5.5em 0 0 0;
  width: 100%;
  -moz-border-radius: 5px 0 0 5px;
  -webkit-border-radius: 5px 0 0 5px;
  border-radius: 5px 0 0 5px;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
  opacity: 0.98;
  filter: alpha(opacity=98);
  -moz-opacity: 0.98;
}
#dashboard_actions a {
  color: white;
  font-size: 110%;
  text-decoration: none;
}
#dashboard_actions h2 {
  font-size: 130%;
}
#dashboard_actions ul {
  list-style: none;
  margin: 3em 1em 1em 1em;
  padding: 0;
  text-align: right;
}
#dashboard_actions ul li {
  margin: 1em 0;
  padding: 0;
}
#dashboard_actions .count {
  margin: 0 0 2em 0;
  text-align: center;
}
#dashboard_actions_content {
  padding: 8px;
  position: fixed;
  width: 18em;
  -moz-box-sizing: border-box;
  -webkit-box-sizing: border-box;
  box-sizing: border-box;
}
#dashboard_sidebar {
  float: left;
  top: 0;
  bottom: 1px;
  position: left;
  width: 18em;
}
#dashboard_sidebar.scrollable {
  float: none;
  overflow-y: auto;
  position: absolute;
}
#dashboard-navbar {
  list-style: none;
  margin: 0;
  padding: 0;
}
#dashboard-navbar li {
  display: block;
  list-style: none;
  margin: 0;
  text-align: left;
  word-break: break-all;
}
#dashboard-navbar li.has-url {
  cursor: pointer;
}
#dashboard-navbar li.has-url:hover .datagrid-sidebar-row,
#dashboard-navbar li.active .datagrid-sidebar-row {
  background: #dcdcdc;
}
#dashboard-navbar li.section {
  margin: 0 0 1em 0;
}
#dashboard-navbar li.section > .datagrid-sidebar-row .label {
  font-size: 110%;
  font-weight: bold;
}
#dashboard-navbar li a {
  color: black;
  text-decoration: none;
  padding: 0;
}
#dashboard-navbar li ul {
  list-style: none;
  margin: 0;
  padding: 0;
}
#dashboard-navbar li ul li.has-count .label {
  /* Give room for the counts. */
  margin-right: 4em;
}
#dashboard-navbar li ul li.new-subsection {
  margin-top: 1em;
}
#dashboard-navbar li ul li .label {
  display: inline-block;
  /* Give room for the icons. */
  margin-left: 20px;
}
#dashboard-navbar li ul li .rb-icon {
  position: absolute;
  top: 0.4em;
  left: 0.8em;
}
#dashboard-navbar li .count {
  position: absolute;
  top: 0.4em;
  right: 0.8em;
  background: #ffffff;
  border: 1px #d0d0d0 solid;
  font-size: 90%;
  padding: 0px 8px;
  margin-top: 0px;
  -moz-border-radius: 12px;
  -webkit-border-radius: 12px;
  border-radius: 12px;
}
#dashboard-navbar li .count.count-zero {
  color: #666666;
  background: #f1f1f1;
  border-color: #dcdcdc;
}
#dashboard-navbar li .datagrid-sidebar-row {
  line-height: 16px;
  padding: 0.4em 0.8em;
  position: relative;
}
#dashboard-navbar li .datagrid-sidebar-row .rb-icon {
  vertical-align: middle;
}
#dashboard-wrapper {
  background-color: #e9e9e9;
  position: relative;
  -moz-border-radius: 5px;
  -webkit-border-radius: 5px;
  border-radius: 5px;
}
#dashboard-navbar #user_page_profile {
  margin-bottom: 1em;
}
#dashboard-navbar #user_page_profile .nickname {
  font-size: 1.4em;
  margin-bottom: 0;
  padding-bottom: 0;
}
#dashboard-navbar #user_page_profile .email,
#dashboard-navbar #user_page_profile .group {
  color: #0000C0;
}
#dashboard-navbar #user_page_profile .fn,
#dashboard-navbar #user_page_profile .email {
  font-size: 1.2em;
}
#dashboard-navbar #user_page_profile .logged-in {
  margin-top: 1.5em;
}
