{% extends "base.html" %}
{% load compressed i18n rbadmintags staticfiles %}

{% block title %}{{title}} | {% trans "Administration" %}{% endblock %}

{% block scripts %}
<script type="text/javascript">window.__admin_media_prefix__ = "{% filter escapejs %}{% static "admin/" %}{% endfilter %}";</script>
{% endblock %}

{% block extrahead %}
{% block blockbots %}<meta name="robots" content="NONE,NOARCHIVE" />{% endblock %}
{% endblock %}

{% block css %}
{%  compressed_css "admin" %}
{%  block extrastyle %}{% endblock %}
<!--[if lte IE 7]><link rel="stylesheet" type="text/css" href="{% block stylesheet_ie %}{% static "admin/css/ie.css" %}{% endblock %}" /><![endif]-->
{%  if LANGUAGE_BIDI %}<link rel="stylesheet" type="text/css" href="{% block stylesheet_rtl %}{% static "admin/css/rtl.css" %}{% endblock %}" />{% endif %}
{% endblock %}

{% block bodytag %}
<body class="admin {% if is_popup %}popup {% endif %}{% block bodyclass %}{% endblock %}">
{% endblock %}

{% block contenttag %}
<div id="content" class="{% block coltype %}colM{% endblock %}">
{% endblock %}

{% block main_content %}
{%  block messages %}
{%   if messages %}
 <ul class="messagelist">
{%    for message in messages %}
  <li{% if message.tags %} class="{{message.tags}}">{% endif %}{{message}}</li>
{%    endfor %}
 </ul>
{%   endif %}
{%  endblock %}

{%  block pretitle %}{% endblock %}

{%  block content_title %}
{%   if title %}<h1 class="title">{{title}}</h1>{% endif %}
{%  endblock %}
{%  block content %}
{%   block object-tools %}{% endblock %}
{{content}}
{%  endblock %}
{%  block sidebar %}{% endblock %}
{% endblock %}

{% block navbar %}
{%  if user.is_authenticated %}
<ul id="navbar">
 <li><a href="{% url 'reviewboard.admin.views.dashboard' %}" class="nav-first">{% trans "Admin Dashboard" %}</a></li>
 <li><a href="{% url 'reviewboard.admin.views.dashboard' %}db/">{% trans "Database" %}</a></li>
 <li><a href="{% url 'djblets.extensions.views.extension_list' %}">{% trans "Extensions" %}</a></li>
</ul>
{%   block subnavbar %}{% endblock %}
{%  endif %}
{% endblock %}

{% block navbar_post %}
{%  if user.is_authenticated %}
<div id="admin-actions" class="clear">
 {% admin_actions %}
</div>
{%  endif %}
{% endblock %}
