from __future__ import unicode_literals


SEQUENCE = [
    'change_descriptions',
    'last_review_timestamp',
    'shipit_count',
    'default_reviewer_repositories',
    'null_repository',
    'localsite',
    'group_incoming_request_count',
    'group_invite_only',
    'group_visible',
    'default_reviewer_local_site',
    'add_issues_to_comments',
    'file_attachments',
    'file_attachment_comment_extra_data',
    'review_request_last_review_activity_timestamp',
    'review_request_depends_on',
    'commit_id',
    'file_attachment_comment_diff_id',
    'rich_text',
    'base_comment_extra_data',
    'unique_together_baseline',
    'extra_data',
    'review_request_issue_counts',
    'group_mailing_list_charfield',
    'review_request_draft_commit_id',
    'review_request_summary_index',
    'review_request_summary_index_manual',
    'split_rich_text',
]
